#pragma NOIV                    // Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//   File:      IRIS9002.c
//   Contents:  Hooks required to implement USB peripheral function.
//              
//   Copyright (c) 2001 Cypress Semiconductor All rights reserved
//-----------------------------------------------------------------------------
#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // SYNCDELAY macro
#include "encoder.h"

void initialize_encoder();
void Encoder_Write(BYTE addr, BYTE reg, BYTE len, BYTE Data);
void Encoder_Read(BYTE addr, BYTE reg, BYTE len);
void Encoder_LED(BYTE addr, BOOL direction);
BYTE Encoder_Status(BYTE addr);

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

#define ENCODER_N 5
BYTE encoder[ENCODER_N] = {0x30, 0x31, 0x32, 0x33, 0x34}; 																													//Encoder Address 
WORD counter[ENCODER_N]  = {0x38B0, 0x4364, 0xFFFF, 0x0C55, 0xFFFF};																																		//Encoder default counter value
BYTE xdata Encoder_Data[17];
BOOL ENCODER_RDY = FALSE;
BOOL ENCODER_INT = FALSE;
BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings

// EZUSB FX2 PORTC i/o...       port NA for 56-pin FX2
 sbit PC0 = IOC ^ 0;					
// sbit PC1 = IOC ^ 1;					
// sbit PC2 = IOC ^ 2;
// sbit PC3 = IOC ^ 3;
// sbit PC4 = IOC ^ 4;
// sbit PC5 = IOC ^ 5;
// sbit PC6 = IOC ^ 6;
// sbit PC7 = IOC ^ 7;



// EZUSB FX2 PORTE is not bit-addressable...

//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init( void )
{ // Called once at startup
	REVCTL = 0x03;
  SYNCDELAY;
  CPUCS = 0x10;                 // CLKSPD[1:0]=10, for 48MHz operation
  SYNCDELAY;
  IFCONFIG = 0xCB;
  SYNCDELAY;
  EP1OUTCFG = (EP1OUTCFG & 0x7F);
	SYNCDELAY;
	EP1INCFG = (EP1INCFG & 0x7F);
	SYNCDELAY;
	EP4CFG = (EP4CFG & 0x7F);
	SYNCDELAY;
	EP8CFG = (EP8CFG & 0x7F);
	SYNCDELAY;
  EP6CFG = (EP6CFG & 0x7F);
  SYNCDELAY;
	EP2CFG = 0xE2;					// EP2 is VALID=1, 0=0, TYPE1=1, TYPE0=0, SIZE=0, 0=0, BUF1=1, BUF0=0 (DIR=IN, TYPE=BULK, SIZE=512, BUF=2x) 0xE2
	SYNCDELAY;

	FIFORESET = 0x80;
	SYNCDELAY;
	FIFORESET = 0x02;
	SYNCDELAY;
	FIFORESET = 0x04;
	SYNCDELAY;
	FIFORESET = 0x06;
	SYNCDELAY;
	FIFORESET = 0x08;

	EP2FIFOCFG = 0x09; //0x00
	SYNCDELAY;
	FIFORESET = 0x00;
	SYNCDELAY;
	EP2AUTOINLENH = 0x00;
	SYNCDELAY;
	EP2AUTOINLENL = 0x10;
	SYNCDELAY;

  // We want to get SOF interrupts
	USBIE |= bmSOF + bmURES;
	SYNCDELAY;
	PORTCCFG &= 0x00;
	SYNCDELAY;
	OEC = 0x00;
	SYNCDELAY;
  Rwuen = TRUE;                 																														// Enable remote-wakeup
	SYNCDELAY;
  I2CTL &= ~bm400KHZ;
	ENCODER_RDY = FALSE;
	EZUSB_InitI2C();
}

void TD_Poll( void )
{ // Called repeatedly while the device is idle
	
	int i, ii, iii;
	BYTE Status;
	
	/*if(!ENCODER_INT){
		while(!PC0);
		while(PC0);
	  ENCODER_INT = TRUE;
	}*/
	
	
	if(!(EP2CS & 0x08)){	
		if (ENCODER_RDY){
			if(!PC0){
			//if (ENCODER_INT){							
				for(i=0; i < ENCODER_N; i++){
					Status = Encoder_Status(encoder[i]);				//will return data from I2C in ENCODER_DATA
					switch(Status)
					{
						case E_INCREMENT:		//0x02
							Encoder_LED(encoder[i], 0x01);
							Encoder_Read(encoder[i], DATA_COUNTER_BYTE2, 2);
							counter[i] = Encoder_Data[1];
							counter[i] |= Encoder_Data[0] << 8;
							break;
						case E_DECREMENT:		//0x04
							Encoder_LED(encoder[i], 0x00);
							Encoder_Read(encoder[i], DATA_COUNTER_BYTE2, 2);
							counter[i] = Encoder_Data[1];
							counter[i] |= Encoder_Data[0] << 8;
							break;							
					}
					Status = 0x00;
				}
			}
			ii=0;
			iii=1;
			for(i=0;i<ENCODER_N;i++){
				EP2FIFOBUF[ii] = LSB(counter[i]);
				EP2FIFOBUF[iii] = MSB(counter[i]);
				ii = ii + 2;
				iii = iii + 2;
			}
			for(i=10;i<16;i++){					//fill the remaining bytes with 0xFF
				EP2FIFOBUF[i] = 0xFF;
			}
			EP2BCH=0x00;
			SYNCDELAY;
			EP2BCL=0x10;
			SYNCDELAY;	
		}
	}
}

BOOL TD_Suspend( void )          
{ // Called before the device goes into suspend mode
   return( FALSE );		//originally True
}

BOOL TD_Resume( void )          
{ // Called after the device resumes
   return( TRUE );
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------
BOOL DR_GetDescriptor( void )
{
	return( TRUE );
}

BOOL DR_SetConfiguration( void )   
{ // Called when a Set Configuration command is received 
  Configuration = SETUPDAT[ 2 ];
  return( TRUE );        // Handled by user code
}

BOOL DR_GetConfiguration( void )   
{ // Called when a Get Configuration command is received
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);          // Handled by user code
}

BOOL DR_SetInterface( void )       
{ // Called when a Set Interface command is received
   return( TRUE );        // Handled by user code
}

BOOL DR_GetInterface( void )       
{ // Called when a Set Interface command is received
   EP0BUF[ 0 ] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return( TRUE );        // Handled by user code
}

BOOL DR_GetStatus( void )
{
   return( TRUE );
}

BOOL DR_ClearFeature( void )
{
   return( TRUE );
}

BOOL DR_SetFeature( void )
{
   return( TRUE );
}


//----------------------------------------------------------------------------
// Conversion of SETUPDAT from EZUSB Vendor Request Tool Bar.
//----------------------------------------------------------------------------

		// SETUPDAT[0] = Direction of Transfer (0x40 = IN, 0xC0 = OUT)
		// SETUPDAT[1] = Req
		// SETUPDAT[2] = LSB of Value
		// SETUPDAT[3] = MSB of Value
		// SETUPDAT[4] = LSB of INDEX
		// SETUPDAT[5] = MSB of INDEX
		// SETUPDAT[6] = LSB of LENGTH (Set to 0 unless doing an A2 or FD request) 
		// SETUPDAT[7] = MSB of LENGTH (Set to 0 unless doing an A2 or FD request) 


#define INIT_ENCODER	0xFE


BOOL DR_VendorCmnd(void)
{
	
  switch(SETUPDAT[1])
	{ //TPM handle new commands
				
		case INIT_ENCODER:
			initialize_encoder();
		break;
		
		default:
				//  Force Renumeration
  		EZUSB_Discon(TRUE);
			break;
    
	}
  	return(FALSE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav( void ) interrupt 0
{
   	GotSUD = TRUE;         // Set flag
   	EZUSB_IRQ_CLEAR( );
   	USBIRQ = bmSUDAV;      // Clear SUDAV IRQ

}

// Setup Token Interrupt Handler
void ISR_Sutok( void ) interrupt 0
{
   	EZUSB_IRQ_CLEAR( );
   	USBIRQ = bmSUTOK;      // Clear SUTOK IRQ
}

void ISR_Sof( void ) interrupt 0
{
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSOF;        // Clear SOF IRQ
}

void ISR_Ures( void ) interrupt 0
{
   if ( EZUSB_HIGHSPEED( ) )
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmURES;       // Clear URES IRQ
}									 

void ISR_Susp( void ) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSUSP;
}

void ISR_Highspeed( void ) interrupt 0
{
   if ( EZUSB_HIGHSPEED( ) )
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack( void ) interrupt 0
{
}
void ISR_Stub( void ) interrupt 0
{
}
void ISR_Ep0in( void ) interrupt 0
{
}
void ISR_Ep0out( void ) interrupt 0
{
}
void ISR_Ep1in( void ) interrupt 0
{
}
void ISR_Ep1out( void ) interrupt 0
{
}
void ISR_Ep2inout( void ) interrupt 0
{	
}
void ISR_Ep4inout( void ) interrupt 0
{
}
void ISR_Ep6inout( void ) interrupt 0
{
}
void ISR_Ep8inout( void ) interrupt 0
{
}
void ISR_Ibn( void ) interrupt 0
{
}
void ISR_Ep0pingnak( void ) interrupt 0
{
}
void ISR_Ep1pingnak( void ) interrupt 0
{
}
void ISR_Ep2pingnak( void ) interrupt 0
{
}
void ISR_Ep4pingnak( void ) interrupt 0
{
}
void ISR_Ep6pingnak( void ) interrupt 0
{
}
void ISR_Ep8pingnak( void ) interrupt 0
{
}
void ISR_Errorlimit( void ) interrupt 0
{
}
void ISR_Ep2piderror( void ) interrupt 0
{
}
void ISR_Ep4piderror( void ) interrupt 0
{
}
void ISR_Ep6piderror( void ) interrupt 0
{
}
void ISR_Ep8piderror( void ) interrupt 0
{
}
void ISR_Ep2pflag( void ) interrupt 0
{
}
void ISR_Ep4pflag( void ) interrupt 0
{
}
void ISR_Ep6pflag( void ) interrupt 0
{
}
void ISR_Ep8pflag( void ) interrupt 0
{
}
void ISR_Ep2eflag( void ) interrupt 0
{
}
void ISR_Ep4eflag( void ) interrupt 0
{
}
void ISR_Ep6eflag( void ) interrupt 0
{
}
void ISR_Ep8eflag( void ) interrupt 0
{
}
void ISR_Ep2fflag( void ) interrupt 0
{
}
void ISR_Ep4fflag( void ) interrupt 0
{
}
void ISR_Ep6fflag( void ) interrupt 0
{
}
void ISR_Ep8fflag( void ) interrupt 0
{
}
void ISR_GpifComplete( void ) interrupt 0
{
}
void ISR_GpifWaveform( void ) interrupt 0
{
}

void initialize_encoder()
{
		int i, ii, iii;
		BYTE Status;
		
		ii = 0;
		iii =1;
//Reset the FIFO
		FIFORESET = 0x80;
		SYNCDELAY;
		FIFORESET = 0x02;
		SYNCDELAY;
		FIFORESET = 0x00;
		SYNCDELAY;
//Fill Encoder_Data with Setup contents
		for (i=0; i < ENCODER_N; i++){
			Encoder_Data[0] = 0x00; 		//Register to Start
			Encoder_Data[1] = 0x0B; 		//General Config
			Encoder_Data[2] = 0x00;			//Status
			Encoder_Data[3] = 0x00;			//Counter Byte 4
			Encoder_Data[4] = 0x00; 		//Counter Byte 3
			Encoder_Data[5] = MSB(counter[i]); 		//Counter Byte 2
			Encoder_Data[6] = LSB(counter[i]); 		//Counter Byte 1
			Encoder_Data[7] = 0x00; 		//Max Byte 4
			Encoder_Data[8] = 0x00; 		//Max Byte 3
			Encoder_Data[9] = 0xFF;			//Max Byte 2
			Encoder_Data[10] = 0xFF; 		//Max Byte 1
			Encoder_Data[11] = 0x00;		//Min Byte 4
			Encoder_Data[12] = 0x00;		//Min Byte 3
			Encoder_Data[13] = 0x00;		//Min Byte 2
			Encoder_Data[14] = 0x00;		//Min Byte 1
			Encoder_Data[15] = 0x00;		//LED A
			Encoder_Data[16] = 0x00;		//LED B		
			EZUSB_WriteI2C(encoder[i], 17, (WORD)Encoder_Data);
			EZUSB_WaitForEEPROMWrite(encoder[i]);
			Status = Encoder_Status(encoder[i]);
			EP2FIFOBUF[ii] = LSB(counter[i]);
			EP2FIFOBUF[iii] = MSB(counter[i]);
			ii = ii + 2;
			iii = iii + 2;
			//Encoder_LED(encoder[i], INCREASE);
		}
		for (i=10;i<16;i++){
			EP2FIFOBUF[i]=0xFF;
		}
		ENCODER_RDY = TRUE;
}
void Encoder_Write(BYTE addr, BYTE reg, BYTE len, BYTE Data)
{
			int i;
			
			for(i=0;i<5;i++)
			{
				Encoder_Data[i]=0x00;
			}
			Encoder_Data[0] = reg;
			if (Data > 0x7F){
				for(i=1 ;i<4 ;i++)
				{
					Encoder_Data[i] = 0xFF;
				}
				Encoder_Data[4] = Data & 0xFF;
			}
			else
			{
				Encoder_Data[len - 1] = Data;
			}
			EZUSB_WriteI2C(addr , len, (WORD)Encoder_Data);					//write the data
}
void Encoder_Read(BYTE addr, BYTE reg, BYTE len)
{

			Encoder_Data[0] = reg;
			EZUSB_WriteI2C(addr, 1, (WORD)Encoder_Data);						//set the correct register to read from
			EZUSB_ReadI2C(addr, len, (WORD)Encoder_Data);								//read register
}
BYTE Encoder_Status(BYTE addr)
{
			Encoder_Data[0] = ENCODER_STATUS;
			EZUSB_WriteI2C(addr, 1, (WORD)Encoder_Data);						//set the correct register to read from
			EZUSB_ReadI2C(addr, 1, (WORD)Encoder_Data);								//read register
			return(Encoder_Data[0]);
}
void Encoder_LED(BYTE addr, BOOL direction)
{
		
		if(direction)
			{
				Encoder_Write(addr, 0x0F, 2, 0xFF);
				EZUSB_Delay(50);
				Encoder_Write(addr, 0x0F, 2, 0x00);
			}
			else
			{
				Encoder_Write(addr, 0x0E, 0x02, 0xFF);
				EZUSB_Delay(50);
				Encoder_Write(addr, 0x0E, 0x02, 0x00);
			}
}